$TITLE Ramsey Model: Dual NLP Formulation

SET     T /1*50/
SET     TFIRST(T), TLAST(T);
TFIRST(T) = YES$(ORD(T) EQ 1);
TLAST(T)  = YES$(ORD(T) EQ CARD(T)); 

SCALAR          G       Growth rate             /0.02/
                R       Interest rate           /0.05/
                K0      Capital-output ratio    /3.00/
                DELTA   Depreciation rate       /0.07/
                Kstock  Capital index           /1.00/

                KVS     Base year capital value share;
                

PARAMETER       QREF(T) Reference quantity path
                PREF(T) Reference price path;

QREF(T) = (1+G)**(ORD(T)-1);
PREF(T) = (1/(1+R))**(ORD(T)-1);

KVS = K0 * (DELTA + R);

PARAMETER ALPHA;
ALPHA(T) = ((1+G)/(1+R))**(ORD(T)-1);
ALPHA(TLAST) = ALPHA(TLAST) / (1-(1+G)/(1+R));

VARIABLE
     OBJ        Objective price;

POSITIVE VARIABLES
     P(T)       Price of output,
     PC(T)      Consumption price
     PK(T)      Price of capital,
     RK(T)      Price of rental capital,
     PL(T)      Wage rate,
     PT         Price of capital in terminal period;

EQUATIONS 
     OBJDEF     Objective function,
     PCDEF(T)   Defines PC
     PRF_Y(T)   Zero profit condition for output,
     PRF_I(T)   Zero profit condition for investment,
     PRF_K(T)   Zero profit condition for capital;


OBJDEF..        OBJ =E= SUM(T, ALPHA(T)*LOG(PC(T)))
               - SUM(T, PL(T) * QREF(T))
               - SUM(TFIRST, PK(TFIRST)* K0 * KSTOCK);

PCDEF(T)..      PC(T) =E= P(T) / PREF(T);

PRF_Y(T)..     (RK(T)/(R+DELTA))**KVS * (PL(T)/(1-KVS))**(1-KVS) =E= P(T);

PRF_I(T)..     P(T) =E= PK(T+1) + PT$TLAST(T);

PRF_K(T)..     PK(T) + (PT*(G+DELTA))$TLAST(T) =E= RK(T) + PK(T+1)*(1-DELTA);

MODEL DUAL /OBJDEF, PCDEF, PRF_Y, PRF_I, PRF_K /;

P.L(T)    = PREF(T);
PC.L(T)   = 1;
PK.L(T)   = (1+R) * PREF(T);
RK.L(T)   = PREF(T) * (DELTA+R);
PL.L(T)   = PREF(T)*(1-KVS);
PT.L      = SUM(TLAST, PREF(TLAST));

PC.LO(T) = 0.001;
P.LO(T)   = 1E-6;
PK.LO(T)  = 1E-6;
RK.LO(T)  = 1E-6;
PL.LO(T)   = 1E-6;
PT.LO     = 1E-6;

SOLVE DUAL USING NLP MAXIMIZING OBJ;


KSTOCK = 0.80;
SOLVE DUAL USING NLP MAXIMIZING OBJ;

PARAMETER       SOLUTION        Solution values;

SOLUTION(T,"PK") = PK.L(T) / P.L(T);
SOLUTION(T,"C") = -PRF_Y.M(T) + PRF_I.M(T);
SOLUTION(T,"I") = -PRF_I.M(T);
SOLUTION(T,"K") = -PRF_K.M(T);

DISPLAY SOLUTION;


