$TITLE Ramsey Model: MCP based on KKT conditions for primal model

SET     T /1*50/
SET     TFIRST(T), TLAST(T);
TFIRST(T) = YES$(ORD(T) EQ 1);
TLAST(T)  = YES$(ORD(T) EQ CARD(T)); 

SCALAR          G       Growth rate             /0.02/
                R       Interest rate           /0.05/
                K0      Capital-output ratio    /3.00/
                DELTA   Depreciation rate       /0.07/
                Kstock  Capital index           /1.00/

                KVS     Base year capital value share
                I0      Base year investment
                C0      Base year consumption
                L0      Base year labor input
                U0      Baseline expenditure;
                

I0 = (DELTA + G) * K0;
C0 = 1 - I0;
L0 = 1 - K0 * (DELTA + R);

PARAMETER       QREF(T) Reference quantity path
                PREF(T) Reference price path;

QREF(T) = (1+G)**(ORD(T)-1);
PREF(T) = (1/(1+R))**(ORD(T)-1);

KVS = K0 * (DELTA + R);

PARAMETER ALPHA;
ALPHA(T) = ((1+G)/(1+R))**(ORD(T)-1);
ALPHA(TLAST) = ALPHA(TLAST) / (1-(1+G)/(1+R));


POSITIVE VARIABLES
     P(T)       Price of output,
     PK(T)      Price of capital,
     RK(T)      Price of rental capital,
     PTC(T)     Price of capital in terminal period
     K(T)       Capital stock
     C(T)       Consumption
     I(T)       Investment;

EQUATIONS
     CC(T)      Capacity constraint,
     KK(T)      Capital balance,
     TC(T)      Terminal condition (provides for post-terminal growth),
     OPT_C(T)   First order optimality condition for C,
     OPT_K(T)   First order optimality condition for K,
     OPT_I(T)   First order optimality condition for I;


CC(T)..         (K(T)/K0)**KVS * QREF(T)**(1-KVS) =E= C(T) + I(T);
KK(T)..         (1-DELTA)*K(T-1) + (K0*KSTOCK)$TFIRST(T) + I(T-1) =G= K(T);
TC(TLAST)..     I(TLAST) =G= (G+DELTA)*K(TLAST);

OPT_C(T)..     C(T)*P(T) =E= C0 * ALPHA(T);

OPT_K(T)..     PK(T) + (PTC(T)*(G+DELTA))$TLAST(T) =E= 
                P(T)*KVS*(K(T)/K0)**KVS * QREF(T)**(1-KVS)/K(T) + PK(T+1)*(1-DELTA);

OPT_I(T)..     P(T) =E= PK(T+1) + PTC(T)$TLAST(T);

MODEL KKT / OPT_C.C, OPT_K.K, OPT_I.I, CC.P, KK.PK, TC.PTC /;

*    Initialize price levels

C.L(T)  = C0 * QREF(T);
I.L(T)  = I0 * QREF(T);
K.L(T)  = K0 * QREF(T);

P.L(T)  = PREF(T);
PK.L(T) = (1+R) * PREF(T);
PTC.L(T) = PREF(T);

PK.LO(T)  = 1E-6;
PK.UP(T)  = +INF;

SOLVE KKT USING MCP;


KSTOCK = 0.8;

SOLVE KKT USING MCP;

PARAMETER       SOLUTION        Solution values;

SOLUTION(T,"PK") = PK.L(T) / P.L(T);
SOLUTION(T,"C") = C.L(T);
SOLUTION(T,"I") = I.L(T);
SOLUTION(T,"K") = K.L(T);

DISPLAY SOLUTION;
