$TITLE  Ramsey Model -- Generates Detailed Plot Output

$if not setglobal termcnd $setglobal termcnd MCP
$if not setglobal tlast   $setglobal tlast 50

$INCLUDE model

PARAMETER       GROWTH  Growth rate for various quantities
                Level   Activity levels 
                Price   Prices ;

$SETGLOBAL gp_ylabel "Percent per annum"

GROWTH(T+1,"L") = 100 * G;
GROWTH(T+1,"K") = 100 * (K.L(T+1)/K.L(T) - 1);
GROWTH(T+1,"Y") = 100 * (Y.L(T+1)/Y.L(T) - 1);
GROWTH(T+1,"I") = 100 * (I.L(T+1)/I.L(T) - 1);
GROWTH(T+1,"C") = 100 * (C.L(T+1)/C.L(T) - 1);
$LIBINCLUDE GNUPLOT GROWTH

$SETGLOBAL gp_ylabel "Percent change from steady-state"
LEVEL(T,"K") = 100 * (K.L(T)/(K0*QREF(T)) - 1);
LEVEL(T,"Y") = 100 * (Y.L(T)/QREF(T) - 1);
LEVEL(T,"I") = 100 * (I.L(T)/(I0*QREF(T)) - 1);
LEVEL(T,"C") = 100 * (C.L(T)/(C0*QREF(T)) - 1);
$LIBINCLUDE GNUPLOT LEVEL

PRICE(T,"PL") = 100 * (PL.L(T)/P.L(T) - 1);
PRICE(T,"RK") = 100 * (RK.L(T)/((DELTA+R)*P.L(T)) - 1);
PRICE(T,"PK") = 100 * (PK.L(T)/((1+R)*P.L(T)) - 1);
$LIBINCLUDE GNUPLOT PRICE

$include ramsey.sol

parameter       error   %termcnd% terminal approximation;

$SETGLOBAL gp_ylabel 

loop(t$(not tlast(t)),

error(T,"I") = 100 * ( solution(T,"I") / ( I.L(T)) - 1);
error(T,"C") = 100 * ( solution(T,"C") / ( C.L(T)) - 1);
error(T,"RK") = 100 * ( solution(T,"RK") / ( RK.L(T)/((DELTA+R)*P.L(T))) - 1);
*.error(T,"PK") = 100 * ( solution(T,"PK") / ( PK.L(T)/((1+R)*P.L(T))) - 1);
error(T,"PL") = 100 * ( solution(T,"PL") / ( PL.L(T)/ P.L(T)) -  1);

);


parameter       errnorm Norm of error;

set solval / i, c, rk, pk, pl/;
errnorm(tlast,"%termcnd%",solval) = sum(t, abs(error(t,solval)) * pref(t)) /
                                    sum(t, pref(t));

errnorm(tlast,"%termcnd%",t) = error(t,"i");



$setglobal gp_ylabel "percentage error"
$setglobal gp_xlabel "year"
$libinclude gnuplot error

