$TITLE Ramsey Model: Primal NLP Formulation

SET     T /1*50/
SET     TFIRST(T), TLAST(T);
TFIRST(T) = YES$(ORD(T) EQ 1);
TLAST(T)  = YES$(ORD(T) EQ CARD(T)); 

SCALAR          G       Growth rate             /0.02/
                R       Interest rate           /0.05/
                K0      Capital-output ratio    /3.00/
                DELTA   Depreciation rate       /0.07/

                KVS     Base year capital value share
                C0      Base year consumption;
                
C0 = 1 - (DELTA + G) * K0;

PARAMETER       QREF(T) Reference quantity path
                PREF(T) Reference price path;

QREF(T) = (1+G)**(ORD(T)-1);
PREF(T) = (1/(1+R))**(ORD(T)-1);

KVS = K0 * (DELTA + R);

PARAMETER ALPHA;
ALPHA(T) = ((1+G)/(1+R))**(ORD(T)-1);
ALPHA(TLAST) = ALPHA(TLAST) / (1-(1+G)/(1+R));

POSITIVE VARIABLES  
     K(T)       Capital stock
     C(T)       Consumption
     I(T)       Investment;

VARIABLES
     UTILITY    Utility function;

EQUATIONS 
     CC(T)      Capacity constraint,
     KK(T)      Capital balance,
     TC(T)      Terminal condition (provides for post-terminal growth),
     UTIL       Discounted log of consumption: objective function;


CC(T)..         (K(T)/K0)**KVS * QREF(T)**(1-KVS) =E= C(T) + I(T);

KK(T+1)..       (1-DELTA)*K(T) + I(T) =G= K(T+1);

TC(TLAST)..     I(TLAST) =G= (G+DELTA)*K(TLAST);

UTIL..          UTILITY =E= -SUM(T, ALPHA(T)*LOG(C(T)));

MODEL PRIMAL /CC, KK, TC, UTIL/;

C.L(T)          = C0;
K.LO(T)         = 1E-4;
C.LO(T)         = 1E-4;
I.LO(T)         = 1E-4;
K.FX(TFIRST)    = K0;

SOLVE PRIMAL MINIMIZING UTILITY USING NLP;

K.FX(TFIRST)    = K0*0.80;
SOLVE PRIMAL MINIMIZING UTILITY USING NLP;

PARAMETER       SOLUTION        Solution values;

SOLUTION(T,"PK") = KK.M(T)/CC.M(T);
SOLUTION(TFIRST,"PK") = -K.M(TFIRST)/CC.M(TFIRST);
SOLUTION(T,"C") = C.L(T);
SOLUTION(T,"I") = I.L(T);
SOLUTION(T,"K") = K.L(T);

DISPLAY SOLUTION;
