$TITLE  Ramsey Model -- MPSGE formulation with alternative termination methods

$if not setglobal termcnd $setglobal termcnd MCP
$if not setglobal tlast   $setglobal tlast 20
$setglobal gp_grid yes
$setglobal gp_yrange [-4:1]

$if exist defines.inc $include defines.inc

SCALAR NLPTERM /0/, MCPTERM/0/;
%termcnd%term = 1;

$setglobal gp_xl tl
SET     T /1*%tlast%/,
        TL(*) /5, 10, 15, 20, 25, 30, 35, 40, 45, 50/;

SET     TFIRST(T)
        TLAST(T);

TFIRST(T) = YES$(ORD(T) EQ 1);
TLAST(T)  = YES$(ORD(T) EQ CARD(T)); 

SCALAR          G       Growth rate             /0.02/
                R       Interest rate           /0.05/
                K0      Capital-output ratio    /3.00/
                DELTA   Depreciation rate       /0.07/
                Kstock  Capital index           /1.00/

                RK0     Base year return to capital
                I0      Base year investment
                C0      Base year consumption
                L0      Base year labor input;
                

I0 = (DELTA + G) * K0;
C0 = 1 - I0;
L0 = 1 - K0 * (DELTA + R);
display l0;

PARAMETER       QREF(T) Reference quantity path
                PREF(T) Reference price path;

QREF(T) = (1+G)**(ORD(T)-1);
PREF(T) = (1/(1+R))**(ORD(T)-1);

PARAMETER ALPHA;
ALPHA(T) = ((1+G)/(1+R))**(ORD(T)-1);
ALPHA(TLAST) = ALPHA(TLAST) / (1-(1+G)/(1+R));

$ONTEXT

$MODEL:RAMSEY

$SECTORS:
        Y(T)    !       Output
        I(T)    !       Investment
        K(T)    !       Capital stock

$COMMODITIES:
        P(T)    !       Output price
        RK(T)   !       Return to capital
        PK(T)   !       Capital price
        PL(T)   !       Wage rate
        PKT     !       Terminal capital

$CONSUMERS:
        RA      !       Representative agent

$AUXILIARY:
        TK$MCPTERM      !       Post-terminal capital stock

$PROD:Y(T) s:1
        O:P(T)          Q:1
        I:PL(T)         Q:L0
        I:RK(T)         Q:K0    P:(DELTA+R)

$PROD:K(T)$MCPTERM
        O:PK(T+1)       Q:(1-DELTA)
        O:PKT$TLAST(T)  Q:(1-DELTA)
        O:RK(T)         Q:1
        I:PK(T)         Q:1

$PROD:K(T)$NLPTERM
        O:PK(T+1)       Q:(1-DELTA)
        O:RK(T)         Q:1
        I:PK(T)         Q:1
        I:PKT$TLAST(T)  Q:(G+DELTA)


$PROD:I(T)
        O:PK(T+1)       Q:1
        O:PKT$TLAST(T)  Q:1
        I:P(T)          Q:1

$DEMAND:RA  s:1
        D:P(T)$MCPTERM  Q:(QREF(T)*C0)  P:PREF(T)
        D:P(T)$NLPTERM  Q:C0            P:ALPHA(T)
        E:PL(T)         Q:(L0*QREF(T))
        E:PK(TFIRST)    Q:(K0*KSTOCK)
        E:PKT$MCPTERM   Q:-1    R:TK

$REPORT:
        V:C(T)  D:P(T)  DEMAND:RA

$CONSTRAINT:TK$MCPTERM
        SUM(T$TLAST(T+1),  I(T+1)/I(T) - Y(T+1)/Y(T)) =E= 0;

$OFFTEXT
$SYSINCLUDE mpsgeset RAMSEY

Y.L(T)  = QREF(T);
I.L(T)  = I0 * QREF(T);
K.L(T)  = K0 * QREF(T);

P.L(T)  = PREF(T);
RK.L(T) = PREF(T) * (DELTA+R);
PK.L(T) = (1+R) * PREF(T);
PL.L(T) = PREF(T);
PKT.L   = SUM(TLAST, PREF(TLAST));

TK.L    = K0 * (1+G)**CARD(T);

RAMSEY.ITERLIM = 0;
$INCLUDE RAMSEY.GEN
SOLVE RAMSEY USING MCP;

KSTOCK = 0.80;

RAMSEY.ITERLIM = 1000;
$INCLUDE RAMSEY.GEN
SOLVE RAMSEY USING MCP;

$include ramsey.sol

parameter       error   %termcnd% terminal approximation;

$SETGLOBAL gp_ylabel 

loop(t$(not tlast(t)),

error(T,"I") = 100 * ( solution(T,"I") / ( I.L(T)) - 1);
error(T,"C") = 100 * ( solution(T,"C") / ( C.L(T)) - 1);
error(T,"RK") = 100 * ( solution(T,"RK") / ( RK.L(T)/((DELTA+R)*P.L(T))) - 1);
*.error(T,"PK") = 100 * ( solution(T,"PK") / ( PK.L(T)/((1+R)*P.L(T))) - 1);
error(T,"PL") = 100 * ( solution(T,"PL") / ( PL.L(T)/ P.L(T)) -  1);

);


parameter       errnorm Norm of error;

set solval / i, c, rk, pk, pl/;
errnorm(tlast,"%termcnd%",solval) = sum(t, abs(error(t,solval)) * pref(t)) /
                                    sum(t, pref(t));

errnorm(tlast,"%termcnd%",t) = error(t,"i");


file kerr /ramsey.err/; kerr.ap = 1; put kerr;
$libinclude gams2txt errnorm


$exit

$setglobal gp_ylabel "percentage error"
$setglobal gp_xlabel "year"
$libinclude gnuplot error


parameter       solution;

solution(T,"K") = K.L(T);
solution(T,"Y") = Y.L(T);
solution(T,"I") = I.L(T);
solution(T,"C") = C.L(T);
solution(T,"PL") = PL.L(T)/P.L(T);
solution(T,"RK") = RK.L(T)/((DELTA+R)*P.L(T));
solution(T,"PK") = PK.L(T)/((1+R)*P.L(T));

file ksol /ramsey.sol/; put ksol;
$libinclude gams2prm solution



PARAMETER       GROWTH  Growth rate for various quantities
                Level   Activity levels 
                Price   Prices ;

$SETGLOBAL gp_ylabel "Percent per annum"

GROWTH(T+1,"L") = 100 * G;
GROWTH(T+1,"K") = 100 * (K.L(T+1)/K.L(T) - 1);
GROWTH(T+1,"Y") = 100 * (Y.L(T+1)/Y.L(T) - 1);
GROWTH(T+1,"I") = 100 * (I.L(T+1)/I.L(T) - 1);
GROWTH(T+1,"C") = 100 * (C.L(T+1)/C.L(T) - 1);
$LIBINCLUDE GNUPLOT GROWTH

$SETGLOBAL gp_ylabel "Percent change from steady-state"
LEVEL(T,"K") = 100 * (K.L(T)/(K0*QREF(T)) - 1);
LEVEL(T,"Y") = 100 * (Y.L(T)/QREF(T) - 1);
LEVEL(T,"I") = 100 * (I.L(T)/(I0*QREF(T)) - 1);
LEVEL(T,"C") = 100 * (C.L(T)/(C0*QREF(T)) - 1);
$LIBINCLUDE GNUPLOT LEVEL

PRICE(T,"PL") = 100 * (PL.L(T)/P.L(T) - 1);
PRICE(T,"RK") = 100 * (RK.L(T)/((DELTA+R)*P.L(T)) - 1);
PRICE(T,"PK") = 100 * (PK.L(T)/((1+R)*P.L(T)) - 1);
$LIBINCLUDE GNUPLOT PRICE


$label getsol

